// Copyright (c) 2022 Dominic Masters
// 
// This software is released under the MIT License.
// https://opensource.org/licenses/MIT

#include "StandardRenderPipeline.hpp"
#include "scene/components/Components.hpp"

using namespace Dawn;

StandardRenderPipeline::StandardRenderPipeline(RenderManager &renderManager) :
  RenderPipeline(renderManager)
{
}

void StandardRenderPipeline::renderScene(Scene &scene) {
  RenderTarget &backBuffer = this->renderManager.getBackBuffer();
  auto cameras = scene.findComponents<Camera>();
  std::shared_ptr<Camera> backBufferCamera = nullptr;

  // First, render all non-backbuffer cameras.
  auto it = cameras.begin();
  while(it != cameras.end()) {
    RenderTarget &cameraTarget = (*it)->getRenderTarget();

    // Leave the backbuffer camera(s) to last, so we skip them.
    if(&cameraTarget == &backBuffer) {
      backBufferCamera = *it;
    } else {
      this->renderSceneCamera(scene, **it);
    }

    ++it;
  }

  // Now render the backbuffer camera.
  if(backBufferCamera == nullptr) return;
  this->renderSceneCamera(scene, *backBufferCamera);
}

void StandardRenderPipeline::renderSceneCamera(Scene &scene, Camera &camera) {
  RenderTarget &renderTarget = camera.getRenderTarget();
  renderTarget.bind();
  renderTarget.clear(
    RENDER_TARGET_CLEAR_FLAG_DEPTH |
    RENDER_TARGET_CLEAR_FLAG_COLOR
  );

  auto meshes = scene.findComponents<MeshRenderer>();
  auto it = meshes.begin();
  while(it != meshes.end()) {
    (*it)->mesh->draw(MESH_DRAW_MODE_TRIANGLES, 0, -1);
    ++it;
  }
}